import 'package:code_chat_ui/model/chat_message.dart';
import 'package:flutter/material.dart';
import 'package:flutter_html/flutter_html.dart';

import 'package:flutter_html_table/flutter_html_table.dart';
import 'package:intl/intl.dart';
import 'package:url_launcher/url_launcher_string.dart';

class ChatMessageWidget extends StatelessWidget {
  final ChatMessage chat;
  final Widget? botIcon;
  const ChatMessageWidget({
    Key? key,
    required this.chat,
    this.botIcon,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Container(
          alignment: Alignment.centerRight,
          child: Container(
            padding: const EdgeInsets.all(15),
            decoration: const BoxDecoration(
              color: Color(0xFFE1E9FF),
              borderRadius: BorderRadius.only(
                topLeft: Radius.circular(20),
                bottomLeft: Radius.circular(20),
                bottomRight: Radius.circular(20),
              ),
            ),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.end,
              children: [
                if (chat.date != null)
                  Padding(
                    padding: const EdgeInsets.only(bottom: 5),
                    child: Text(
                      DateFormat('dd MMM yyyy HH:mm').format(chat.date!),
                      style: TextStyle(
                        fontSize: 12,
                        color: Colors.grey[600],
                      ),
                    ),
                  )
                else
                  const SizedBox(),
                Text(
                  chat.user,
                ),
              ],
            ),
          ),
        ),
        const SizedBox(
          height: 10,
        ),
        Row(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            if (botIcon != null)
              Padding(
                padding: const EdgeInsets.only(right: 10),
                child: botIcon,
              ),
            Expanded(
              child: Container(
                alignment: Alignment.centerLeft,
                child: Container(
                  padding: const EdgeInsets.all(15),
                  alignment: Alignment.centerLeft,
                  decoration: BoxDecoration(
                    color: Colors.grey[300],
                    borderRadius: const BorderRadius.only(
                      topRight: Radius.circular(20),
                      bottomLeft: Radius.circular(20),
                      bottomRight: Radius.circular(20),
                    ),
                  ),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      if (chat.date != null)
                        Padding(
                          padding: const EdgeInsets.only(bottom: 5),
                          child: Text(
                            DateFormat('dd MMM yyyy HH:mm').format(chat.date!),
                            style: TextStyle(
                              fontSize: 12,
                              color: Colors.grey[600],
                            ),
                          ),
                        )
                      else
                        const SizedBox(),
                      Builder(
                        builder: (context) {
                          final text = chat.bot;
                          // // regex to find url in text
                          RegExp exp = RegExp(
                              r'(?:(?:https?|ftp):\/\/)?[\w/\-?=%.]+\.[\w/\-?=%.]+');
                          // regex to find a href link in text
                          RegExp expHref = RegExp(
                              r'<a href=(?:(?:https?|ftp):\/\/)?[\w/\-?=%.]+\.[\w/\-?=%.]');

                          // newText is text with <br> instead of new line
                          String newText = text.replaceAll('\n', '<br>');

                          // if text contains a href tag, remove the tag except the url inside the tag
                          if (!expHref.hasMatch(newText)) {
                            newText = newText.replaceAllMapped(
                              exp,
                              (match) {
                                final url = match.group(0);
                                return '<a href="$url">$url</a>';
                              },
                            );
                          }

                          return Html(
                            data: newText,
                            onLinkTap: (url, attributes, element) async {
                              if (url == null) return;
                              if (await canLaunchUrlString(url)) {
                                launchUrlString(url);
                              }
                            },
                            extensions: const [
                              TableHtmlExtension(),
                            ],
                            style: {
                              "body": Style(
                                fontSize: FontSize(14),
                              ),
                              "a": Style(
                                color: Colors.blue,
                              ),
                            },
                          );
                        },
                      ),
                    ],
                  ),
                ),
              ),
            ),
          ],
        ),
        const SizedBox(
          height: 10,
        ),
      ],
    );
  }
}
